/******************************************************************************/
/* idt.c  -- see http://www.zeppoo.net                                        */
/*                                                                            */
/* The project zeppoo is (C) 2006 : contact@zeppoo.net                        */
/* This program is free software;                                             */
/* you can redistribute it and/or modify it under the terms of the GNU        */
/* General Public License as published by the Free Software Foundation;       */
/* Version 2. This guarantees your right to use, modify, and                  */
/* redistribute this software under certain conditions.                       */
/*                                                                            */
/* Source is provided to this software because we believe users have a        */
/* right to know exactly what a program is going to do before they run        */
/* it.                                                                        */
/*                                                                            */
/* This program is distributed in the hope that it will be                    */
/* useful, but WITHOUT ANY WARRANTY; without even the implied                 */
/* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR                    */
/* PURPOSE. See the GNU General Public License for more details (             */
/* http://www.gnu.org/copyleft/gpl.html ).                                    */
/*                                                                            */
/******************************************************************************/

#include "idt.h"

void myfree_didt(struct descriptor_idt *tmp){
	free(tmp);
}


void getIdtMemory(ZHTable_t *myidt, char *file){
	zeppoo_get_idt(myidt);
	zeppoo_get_idt_opcodes(myidt);
	if(file == NULL)
		zeppoo_resolve_idt(myidt, DEFAULTSYSTEMMAP);
	else
		zeppoo_resolve_idt(myidt, file);
}

void getIdtFingerprints(FILE *input, ZHTable_t *myidt){
	char key[KEYSIZE];
	char line[80];
	char *pos, *stub_addr, *name, *op;
	struct descriptor_idt *tmp_didt;
	
	fgets(line, 80, input);
	while(strcmp(line,"[END IDT]\n")){
		pos = strtok(line, " ");
		stub_addr = strtok(NULL, " ");
		name = strtok(NULL, " ");
		op = strtok(NULL, " ");	
		
		tmp_didt = (struct descriptor_idt *)malloc(sizeof(struct descriptor_idt));
		tmp_didt->pos = atoi(pos);
		tmp_didt->stub_addr = strtoul(stub_addr, NULL, 16);
		memcpy(tmp_didt->name, name, sizeof(tmp_didt->name));
		tmp_didt->opcodes[0] = strtoul(op, NULL, 16);
		op = strtok(NULL, " ");
		tmp_didt->opcodes[1] = strtoul(op, NULL, 16);

		bzero(key, KEYSIZE);
		snprintf(key, KEYSIZE, "%d", tmp_didt->pos);
		zeppoo_hash_insert(myidt, key, KEYSIZE, tmp_didt, (void *)myfree_didt);
	
		fgets(line, 80, input);
	}
}

void simpleViewIdt(ZHTable_t *myidt){
        struct Item_ZHTable * tmp;
        struct descriptor_idt *tmpdidt;
        printf("POS\t MEM\t\t NAME\t\t\t\t  OPCODES\n");

        tmp = myidt->first;
        while(tmp != NULL){ 
                tmpdidt = (struct descriptor_idt *)tmp->el->value;
                printf("%d\t 0x%x\t %-15s\t\t %8x %8x\n", tmpdidt->pos, (unsigned int)tmpdidt->stub_addr, tmpdidt->name, (unsigned int)tmpdidt->opcodes[0], (unsigned int)tmpdidt->opcodes[1]);
                 
                tmp = tmp->next;
        }
	
}

void viewIdtMemory(char *file){
	ZHTable_t *idtmemory;
	idtmemory = zeppoo_hash_new();

	getIdtMemory(idtmemory, file);
	simpleViewIdt(idtmemory);	
	
	zeppoo_hash_delete(idtmemory);
}

void writeIdtMemory(FILE *output, char *file){
	ZHTable_t *idtmemory;
	struct Item_ZHTable * tmp;
	struct descriptor_idt *tmpdidt;
					      
	idtmemory = zeppoo_hash_new();

	getIdtMemory(idtmemory, file);
	
	printf("\t++ Begin : Generating IDT Fingerprints\n");
	
	tmp = idtmemory->first;
	while(tmp != NULL){
		tmpdidt = (struct descriptor_idt *)tmp->el->value;
		fprintf(output,"%d 0x%x %s %8x %8x\n", tmpdidt->pos, (unsigned int)tmpdidt->stub_addr, tmpdidt->name, (unsigned int)tmpdidt->opcodes[0], (unsigned int)tmpdidt->opcodes[1]);
							                
		tmp = tmp->next;
	}			

	printf("\t++ End : Generating Syscalls Fingerprints\n");
	
	zeppoo_hash_delete(idtmemory);
}

void viewHijackIdt(FILE *input, char *file){
	char key[KEYSIZE];
	ZHTable_t *idtmemory = NULL;
	ZHTable_t *idtfingerprints = NULL;
	ZHTable_t *hijackidt = NULL;
	
	struct Item_ZHTable * tmp;   
	struct descriptor_idt *didtmem, *didtfing, *hijackdidt;
	ZHash_t *hdidt;
	
	idtmemory = zeppoo_hash_new();
	idtfingerprints = zeppoo_hash_new();
	hijackidt = zeppoo_hash_new();
	
	getIdtMemory(idtmemory, file);
	getIdtFingerprints(input, idtfingerprints);

	printf("\t++ Begin : Checking IDT Fingerprints\n");
	
	tmp = idtfingerprints->first;
	while(tmp != NULL){
		bzero(key, KEYSIZE);
		didtfing = (struct descriptor_idt *)tmp->el->value;	
		snprintf(key, KEYSIZE, "%d", didtfing->pos);
		
		hdidt = (ZHash_t *)zeppoo_get_data(idtmemory, key, KEYSIZE);
		didtmem = (struct descriptor_idt *)hdidt->value;
	
		if((didtfing->stub_addr != didtmem->stub_addr) || (didtfing->opcodes[0] != didtmem->opcodes[0]) || (didtfing->opcodes[1] != didtmem->opcodes[1])){           
			hijackdidt = (struct descriptor_idt *)malloc(sizeof(struct descriptor_idt));
			memcpy(hijackdidt->name, didtfing->name, sizeof(hijackdidt->name));
			hijackdidt->pos = didtfing->pos;
			hijackdidt->stub_addr = didtfing->stub_addr;
			hijackdidt->opcodes[0] = didtfing->opcodes[0];
			hijackdidt->opcodes[1] = didtfing->opcodes[1];
			zeppoo_hash_insert(hijackidt, key, KEYSIZE, hijackdidt, (void *)myfree_didt);
							
			
		}
		tmp = tmp->next;
	}

	printf("\t++ End : Checking IDT Fingerprints\n\n");
	
	if(hijackidt->first != NULL){
		printf("LIST OF HIJACK IDT\n");
		simpleViewIdt(hijackidt);
		printf("\n");
	}
	else
		printf("NO HIJACK IDT\n\n");

	zeppoo_hash_delete(idtmemory);
	zeppoo_hash_delete(idtfingerprints);
	zeppoo_hash_delete(hijackidt);
}
