/******************************************************************************/
/* hash.c  -- see http://www.zeppoo.net                                       */
/* 									      */
/* The project zeppoo is (C) 2006 : contact@zeppoo.net                        */
/* This program is free software;                                             */
/* you can redistribute it and/or modify it under the terms of the GNU        */
/* General Public License as published by the Free Software Foundation;       */
/* Version 2. This guarantees your right to use, modify, and                  */
/* redistribute this software under certain conditions.                       */
/*                                                                            */
/* Source is provided to this software because we believe users have a        */
/* right to know exactly what a program is going to do before they run        */
/* it.                                                                        */
/*                                                                            */
/* This program is distributed in the hope that it will be                    */
/* useful, but WITHOUT ANY WARRANTY; without even the implied                 */
/* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR                    */
/* PURPOSE. See the GNU General Public License for more details (             */
/* http://www.gnu.org/copyleft/gpl.html ).                                    */
/*                                                                            */
/******************************************************************************/

#include "zeppoo.h"

/* HASH BASED ON LIBCX */
/* http://www.capflam.org/?page_id=4 */


unsigned long fnv_32a_buf(const void *buf, const unsigned int len, unsigned long hval){
	unsigned char *bp = (unsigned char *)buf;
	unsigned char *be = bp + len;
  

	while (bp < be){
      		hval ^= (unsigned long)*bp++; 
      		hval += (hval<<1) + (hval<<4) + (hval<<7) + (hval<<8) + (hval<<24);
    	}
  	return hval;
}

ZHTable_t *zeppoo_hash_new(){
	ZHTable_t     *zhtable;

	zhtable = (struct ZHTable *)malloc(sizeof(struct ZHTable));
	zhtable->first = zhtable->last = zhtable->current = NULL;

	return zhtable;
}

void zeppoo_hash_delete(ZHTable_t *zhtable){
	struct Item_ZHTable *tmp1, *tmp2;
	
	if (zhtable == NULL)
		return;

	
	tmp1 = zhtable->first;
	while(tmp1){
		tmp2 = tmp1->next;
		if (tmp1->el != NULL){
			tmp1->el->free_value(tmp1->el->value);
			free(tmp1->el);
		}
		free(tmp1);
		tmp1 = tmp2;
	}
	free(zhtable);
}

void insert_key_zhtable(ZHTable_t *zhtable, const unsigned long key, void *value, void (*free_value)(void *)){
	ZHash_t       *zhash;
	struct Item_ZHTable *item;
	
	if (zhtable == NULL || value == NULL || !key)
		return;

	for(item = zhtable->first; item != NULL; item = item->next){
		zhash = item->el;
		if (zhash->key == key)
			return;
	}

	zhash = (ZHash_t *)malloc(sizeof(ZHash_t));
	zhash->value = NULL;
	
	zhash->key = key;
	zhash->value = value;
	zhash->free_value = free_value;

	if(zhash != NULL){
		item = (struct Item_ZHTable *)malloc(sizeof(struct Item_ZHTable));
		item->el = zhash;

		if(zhtable->first == NULL){
			zhtable->first = item;
			zhtable->current = zhtable->first;
		}
		else
			zhtable->last->next = item;

		item->next = NULL;
		zhtable->last = item;
	}
}

void zeppoo_hash_insert(ZHTable_t *zhtable, const void *key, const unsigned int size, void *value, void (*free_value)(void *)){
	unsigned long	hash_key;
	if (zhtable == NULL || key == NULL ||  value == NULL || !size)
		return;
	hash_key = fnv_32a_buf(key, size, 0);
	insert_key_zhtable(zhtable, hash_key, value, free_value);
}


const ZHash_t *get_key_htable(ZHTable_t *zhtable, const unsigned long key){
	ZHash_t *zhash;
	struct Item_ZHTable *item;
	
	if (zhtable == NULL || !key)
		return NULL;

	for(item = zhtable->first; item != NULL; item = item->next){
		zhash = item->el;
		if (zhash->key == key)
			return zhash;
		
	}
	return NULL;
}

const ZHash_t *zeppoo_get_data(ZHTable_t *zhtable, const void *key, const unsigned int size){
	unsigned long hash_key;
	if (zhtable == NULL || key == NULL || !size)
		return NULL;

	hash_key = fnv_32a_buf(key, size, 0);
	return get_key_htable(zhtable, hash_key);
}
