/******************************************************************************/
/* zeppoo.h  -- see http://www.zeppoo.net                                     */
/*                                                                            */
/* The project zeppoo is (C) 2006 : contact@zeppoo.net                        */
/* This program is free software;                                             */
/* you can redistribute it and/or modify it under the terms of the GNU        */
/* General Public License as published by the Free Software Foundation;       */
/* Version 2. This guarantees your right to use, modify, and                  */
/* redistribute this software under certain conditions.                       */
/*                                                                            */
/* Source is provided to this software because we believe users have a        */
/* right to know exactly what a program is going to do before they run        */
/* it.                                                                        */
/*                                                                            */
/* This program is distributed in the hope that it will be                    */
/* useful, but WITHOUT ANY WARRANTY; without even the implied                 */
/* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR                    */
/* PURPOSE. See the GNU General Public License for more details (             */
/* http://www.gnu.org/copyleft/gpl.html ).                                    */
/*                                                                            */
/******************************************************************************/

#ifndef ZEPPOO_H
#define ZEPPOO_H

#define _GNU_SOURCE

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <ctype.h>


/******************* GLOBAL **************************/

#define DEFAULTSYSTEMMAP "/boot/System.map"

struct {
	unsigned short limit;
	unsigned int base;
} __attribute__((packed)) idtr;

struct {
	unsigned short off1;
	unsigned short sel;
	unsigned char none,flags;
	unsigned short off2;
} __attribute__ ((packed)) idt;

void getOpcodes(unsigned long, unsigned long *);
int resolve(char *, unsigned long, char *, int);

/******************* HASH ****************************/

struct ZHash{
	unsigned long key;
	void *value;
	void  (*free_value)(void *);
};

typedef struct ZHash ZHash_t;

struct Item_ZHTable {
        struct ZHash *el;
	struct Item_ZHTable *next;
};

struct ZHTable {
	struct Item_ZHTable *first, *last, *current;
};

typedef struct ZHTable ZHTable_t;


unsigned long fnv_32a_buf(const void *, const unsigned int,  unsigned long);
ZHTable_t *zeppoo_hash_new(void);
void zeppoo_hash_delete(ZHTable_t *);
void insert_key_zhtable(ZHTable_t *, const unsigned long, void *, void (*)(void *));
void zeppoo_hash_insert(ZHTable_t *, const void *, const unsigned int, void *, void (*)(void *));
const ZHash_t *get_key_htable(ZHTable_t *, const unsigned long);
const ZHash_t *zeppoo_get_data(ZHTable_t *, const void *, const unsigned int);

/******************* VERSION ************************/

/* I386 */
#define OP_I386_JMP  "\xe9"
#define OP_I386_CALL "\xe8"
#define OP_I386_CMP  "\x81"

#define KERNEL_I386_START    0xc0100000      
#define KERNEL_I386_END      0xc1000000

struct version {
	float kernel;
	unsigned long kernel_start;     
	unsigned long kernel_end;
	char archi[5];
	char op_jmp[2];					        
	char op_call[2];
	char op_cmp[2];
};

struct version zepversion;

void zeppoo_init_version(char *, char *);

/****************** MEMORY **************************/
int mem;
void *ptr;
int valmmap;

void zeppoo_init_memory(char *, int);

int zeppoo_open_memory();
int zeppoo_close_memory();
int zeppoo_read_memory(unsigned long, void *, int);
int zeppoo_fread_memory(unsigned long, void *, int);
int zeppoo_write_memory(unsigned long, void *, int);
int zeppoo_fwrite_memory(unsigned long, void *, int);


int (*openvMem)();
int (*closevMem)();
int (*readvMem)(unsigned long, void *, int);
int (*writevMem)(unsigned long, void *, int);

/******************* KMEM *************************/

int openkmem();
int closekmem();
int readkmem(unsigned long, void *, int);
int writekmem(unsigned long, void *, int);

/******************* MEM ************************/

#define PAGE_OFFSET 0xc0000000
#define PAGE_MAX    0xffffffff

int openmem();
int closemem();
int readmem(unsigned long, void *, int);
int writemem(unsigned long, void *, int);

/****************** SYMBOLS *********************/

#define PROC_ROOT_INO           0x01
#define PROC_ROOT_NOTHING       0x00
#define PROC_ROOT_NAMELEN       0x05
#define PROC_ROOT_MODE          0x6d

unsigned long zeppoo_find_init_task(void);

/****************** TASKS ***********************/

#define ADDRMIN         0xc0000000
#define DIFFADDR        0x10
#define JMPFIVEADDR     0x14

#define KEYSIZE 8

struct taskInfo{
	unsigned long init_task;
	unsigned long first_addr;
	int offset_name;
	int offset_list;
	int offset_next;
	int offset_pid;
	int offset_uid;
};

struct taskInfo zeptaskinfo;

struct task{
	char name[16];
	int pid;
	int uid;
	int gid;
	int rec;
	unsigned long addr;
};

void free_task(struct task *);

void zeppoo_init_taskInfo(void);
void zeppoo_get_tasks(ZHTable_t *);
int zeppoo_get_task(int, struct task *);

/********************* SYSCALLS *********************/

struct syscall{
	char name[64];
	int pos;
	unsigned long addr;
	unsigned long opcodes[2];
};

void free_syscall(struct syscall *);

int zeppoo_get_syscalls(ZHTable_t *);
int zeppoo_resolve_syscalls(ZHTable_t *, char *file);
void zeppoo_get_syscalls_opcodes(ZHTable_t *);
void getOpcodes(unsigned long , unsigned long *);

/******************** IDT ****************************/

struct descriptor_idt{
	char name[64];
	int pos;
	unsigned long stub_addr;
	unsigned long opcodes[2];
};

void free_didt(struct descriptor_idt *);

int zeppoo_get_idt(ZHTable_t *);
int zeppoo_resolve_idt(ZHTable_t *, char *file);
void zeppoo_get_idt_opcodes(ZHTable_t *);

#endif
