/******************************************************************************/
/* syscalls.c  -- see http://www.zeppoo.net                                   */
/*                                                                            */
/* The project zeppoo is (C) 2006 : contact@zeppoo.net                        */
/* This program is free software;                                             */
/* you can redistribute it and/or modify it under the terms of the GNU        */
/* General Public License as published by the Free Software Foundation;       */
/* Version 2. This guarantees your right to use, modify, and                  */
/* redistribute this software under certain conditions.                       */
/*                                                                            */
/* Source is provided to this software because we believe users have a        */
/* right to know exactly what a program is going to do before they run        */
/* it.                                                                        */
/*                                                                            */
/* This program is distributed in the hope that it will be                    */
/* useful, but WITHOUT ANY WARRANTY; without even the implied                 */
/* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR                    */
/* PURPOSE. See the GNU General Public License for more details (             */
/* http://www.gnu.org/copyleft/gpl.html ).                                    */
/*                                                                            */
/******************************************************************************/

#include "syscalls.h"

void myfree_syscall(struct syscall *tmp){
	free(tmp);
}


void getSyscallsMemory(ZHTable_t *mysyscalls, char *file){
	zeppoo_get_syscalls(mysyscalls);
	zeppoo_get_syscalls_opcodes(mysyscalls);
	if(file == NULL)
		zeppoo_resolve_syscalls(mysyscalls, DEFAULTSYSTEMMAP);
	else 
		zeppoo_resolve_syscalls(mysyscalls, file);
}

void getSyscallsFingerprints(FILE *input, ZHTable_t *mysyscalls){
	char key[KEYSIZE];
	char line[80];
	char *pos, *addr, *name, *op;
	struct syscall *tmp_syscall;
	
	fgets(line, 80, input);
	while(strcmp(line,"[END SYSCALLS]\n")){
		pos = strtok(line, " ");
		addr = strtok(NULL, " ");
		name = strtok(NULL, " ");
		op = strtok(NULL, " ");	
		
		tmp_syscall = (struct syscall *)malloc(sizeof(struct syscall));
		tmp_syscall->pos = atoi(pos);
		tmp_syscall->addr = strtoul(addr, NULL, 16);
		memcpy(tmp_syscall->name, name, sizeof(tmp_syscall->name));
		tmp_syscall->opcodes[0] = strtoul(op, NULL, 16);
		op = strtok(NULL, " ");
		tmp_syscall->opcodes[1] = strtoul(op, NULL, 16);

		bzero(key, KEYSIZE);
		snprintf(key, KEYSIZE, "%d", tmp_syscall->pos);
		zeppoo_hash_insert(mysyscalls, key, KEYSIZE, tmp_syscall, (void *)myfree_syscall);
	
		fgets(line, 80, input);
	}
}

void simpleViewSyscalls(ZHTable_t *mysyscalls){
        struct Item_ZHTable * tmp;
        struct syscall *tmpsysc;
        printf("POS\t MEM\t\t NAME\t\t\t\t  OPCODES\n");

        tmp = mysyscalls->first;
        while(tmp != NULL){ 
                tmpsysc = (struct syscall *)tmp->el->value;
                printf("%d\t 0x%x\t %-15s\t\t %8x %8x\n", tmpsysc->pos, (unsigned int)tmpsysc->addr, tmpsysc->name, (unsigned int)tmpsysc->opcodes[0], (unsigned int)tmpsysc->opcodes[1]);
                 
                tmp = tmp->next;
        }
	
}

void viewSyscallsMemory(char *file){
	ZHTable_t *syscallsmemory;
	syscallsmemory = zeppoo_hash_new();

	getSyscallsMemory(syscallsmemory, file);
	simpleViewSyscalls(syscallsmemory);	
	
	zeppoo_hash_delete(syscallsmemory);
}

void writeSyscallsMemory(FILE *output, char *file){
	ZHTable_t *syscallsmemory;
	struct Item_ZHTable * tmp;
	struct syscall *tmpsysc;
					      
	syscallsmemory = zeppoo_hash_new();

	getSyscallsMemory(syscallsmemory, file);	

	printf("\t++ Begin : Generating Syscalls Fingerprints\n");
	
	tmp = syscallsmemory->first;
	while(tmp != NULL){
		tmpsysc = (struct syscall *)tmp->el->value;
		fprintf(output,"%d 0x%x %s %8x %8x\n", tmpsysc->pos, (unsigned int)tmpsysc->addr, tmpsysc->name, (unsigned int)tmpsysc->opcodes[0], (unsigned int)tmpsysc->opcodes[1]);
							                
		tmp = tmp->next;
	}					

	printf("\t++ End : Generating Syscalls Fingerprints\n");
	
	zeppoo_hash_delete(syscallsmemory);
}

void viewHijackSyscalls(FILE *input, char *file){
	char key[KEYSIZE];
	ZHTable_t *syscallsmemory = NULL;
	ZHTable_t *syscallsfingerprints = NULL;
	ZHTable_t *hijacksyscalls = NULL;
	
	struct Item_ZHTable * tmp;   
	struct syscall *syscmem, *syscfing, *hijacksysc;
	ZHash_t *hsyscall;
	
	syscallsmemory = zeppoo_hash_new();
	syscallsfingerprints = zeppoo_hash_new();
	hijacksyscalls = zeppoo_hash_new();
	
	getSyscallsMemory(syscallsmemory, file);
	getSyscallsFingerprints(input, syscallsfingerprints);
	
	printf("\t++ Begin : Checking Syscalls Fingerprints\n");
	
	tmp = syscallsfingerprints->first;
	while(tmp != NULL){
		bzero(key, KEYSIZE);
		syscfing = (struct syscall *)tmp->el->value;	
		snprintf(key, KEYSIZE, "%d", syscfing->pos);
		
		hsyscall = (ZHash_t *)zeppoo_get_data(syscallsmemory, key, KEYSIZE);
		syscmem = (struct syscall *)hsyscall->value;
	
		if((syscfing->addr != syscmem->addr) || (syscfing->opcodes[0] != syscmem->opcodes[0]) || (syscfing->opcodes[1] != syscmem->opcodes[1])){           
			hijacksysc = (struct syscall *)malloc(sizeof(struct syscall));
			memcpy(hijacksysc->name, syscfing->name, sizeof(hijacksysc->name));
			hijacksysc->pos = syscfing->pos;
			hijacksysc->addr = syscfing->addr;
			hijacksysc->opcodes[0] = syscfing->opcodes[0];
			hijacksysc->opcodes[1] = syscfing->opcodes[1];
			zeppoo_hash_insert(hijacksyscalls, key, KEYSIZE, hijacksysc, (void *)myfree_syscall);
							
			
		}
		tmp = tmp->next;
	}
	printf("\t++ End : Checking Syscalls Fingerprints\n\n");
	
	if(hijacksyscalls->first != NULL){
		printf("LIST OF HIJACK SYSCALLS\n");
		simpleViewSyscalls(hijacksyscalls);
		printf("\n");
	}
	else
		printf("NO HIJACK SYSCALL\n\n");

	zeppoo_hash_delete(syscallsmemory);
	zeppoo_hash_delete(syscallsfingerprints);
	zeppoo_hash_delete(hijacksyscalls);
}
